program ndot
    use pca_futils
    implicit none

    integer, parameter :: n=10000000
    double precision, allocatable:: x(:),y(:)
    integer :: i 
    integer :: timer
    double precision    :: res
    real    :: time
    double precision    :: ans 

    allocate(x(n),y(n))

    do i=1,n
       x(i) = i-1
       y(i) = i-1
    enddo
    ans=n*(n+1.)*(2.*n+1)/6.0

    call tick(timer) 
    res = calc_ndot(n,x,y)
    time = tocksilent(timer)
    print *, 'Dot product is ', res, '(vs ', ans,') for n = ',n,'.  Took ', time, 'sec.'
 
    deallocate(x,y)

    contains

    double precision function calc_ndot(n, x, y)
        implicit none
        integer, intent(in) :: n
        double precision, dimension(n)  :: x
        double precision, dimension(n)  :: y
        double precision :: ndot
        integer :: i
        double precision :: mysum

        ndot = 0.
!$omp parallel default(none) shared(ndot,n,x,y) private(i,mysum)
        mysum = 0
!$omp do
        do i=1,n
            mysum = mysum + x(i)*y(i)
        enddo
!$omp atomic
        ndot = ndot + mysum
!$omp end parallel 
        calc_ndot = ndot
    end function calc_ndot

end program ndot
